#!/usr/bin/env node
'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _metadata = require('./metadata');

var _pipeline = require('./pipeline');

var _loadFile = require('./loadFile');

var _loadFile2 = _interopRequireDefault(_loadFile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var _process$argv = _toArray(process.argv);

var uuid = _process$argv[2];

var parts = _process$argv.slice(3);

var _parts = _toArray(parts);

var command = _parts[0];

var args = _parts.slice(1);

switch (command) {
  case 'meta-data':
    {
      handleMetadata();
      break;
    }
  case 'pipeline':
    {
      handlePipeline();
      break;
    }
  default:
    {
      console.error('bkrun-agent does not implement', command);
      process.exit(1);
    }
}

function handlePipeline() {
  var _args = _slicedToArray(args, 3);

  var upload = _args[0];
  var file = _args[1];
  var replace = _args[2];


  if (file === '--replace') {
    replace = '--replace';
    file = null;
  }

  if (upload !== 'upload') {
    console.error('bkrun-agent pipeline action must be upload');
    process.exit(1);
  }

  function gotPipeline(pipeline) {
    (0, _pipeline.writePipeline)(uuid, {
      replace: replace === '--replace',
      pipeline: pipeline
    });
    process.exit();
  }

  if (file) {
    var pipeline = (0, _loadFile2.default)(_path2.default.resolve(process.cwd(), file));
    return gotPipeline(pipeline);
  }

  process.stdin.setEncoding('utf8');

  var chunks = [];

  process.stdin.on('readable', function () {
    var chunk = process.stdin.read();
    if (chunk) {
      chunks.push(chunk);
    }
  });

  process.stdin.on('end', function () {
    var chunkStr = chunks.join('');
    try {
      return gotPipeline(JSON.parse(chunkStr));
    } catch (e) {
      throw e;
    }
  });
}

function handleMetadata() {
  var _args2 = _slicedToArray(args, 3);

  var action = _args2[0];
  var key = _args2[1];
  var value = _args2[2];


  if (!~['get', 'set'].indexOf(action)) {
    console.error('bkrun-agent meta-data action must be get or set');
    process.exit(1);
  }

  if (action === 'get') {
    try {
      console.log((0, _metadata.getMetadata)(uuid, key));
    } catch (e) {
      process.exit(1);
    }
  }

  if (action === 'set') {
    try {
      (0, _metadata.setMetadata)(uuid, key, value);
    } catch (e) {
      throw e;
    }
  }
}