#!/usr/bin/env node
'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _ioSpin = require('io-spin');

var _ioSpin2 = _interopRequireDefault(_ioSpin);

var _merge = require('merge');

var _merge2 = _interopRequireDefault(_merge);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

require('colors');

var _loadFile = require('./loadFile');

var _loadFile2 = _interopRequireDefault(_loadFile);

var _promptForConfirm = require('./promptForConfirm');

var _promptForConfirm2 = _interopRequireDefault(_promptForConfirm);

var _metadata = require('./metadata');

var _pipeline = require('./pipeline');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var buildId = (0, _uuid2.default)().replace(/-/g, '');

var _process$argv = _slicedToArray(process.argv, 4);

var fileName = _process$argv[2];
var _process$argv$ = _process$argv[3];
var startIndex = _process$argv$ === undefined ? 0 : _process$argv$;


if (!fileName) {
  console.error('Please run again with a filename');
  process.exit(1);
}

var filePath = _path2.default.resolve(process.cwd(), fileName);

var file = (0, _loadFile2.default)(filePath);

if (!file.steps || file.steps.length === 0) {
  console.error('Oops, your pipeline doesn\'t seem to have any steps!');
  process.exit(1);
}

if (!Array.isArray(file.steps)) {
  console.error('Oops, steps needs to be an array!');
  process.exit(1);
}

function processStep(i) {
  var step = file.steps[i];

  if (!step.type) {
    if (step.command) {
      step.type = 'command';
    } else if (step.prompt || step.block) {
      step.type = 'prompt';
    }
  }

  step = rewriteStep(step);

  function finish(err) {
    if (err) {
      // TODO: handle better
      throw err;
    }
    var nextIndex = i + 1;
    if (nextIndex === file.steps.length) {
      // finish
      process.exit();
    } else {
      return processStep(nextIndex);
    }
  }

  switch (step.type) {
    case 'command':
      {
        runCommand(i, step, finish);
        break;
      }
    case 'waiter':
      {
        handleWait(step, finish);
        break;
      }
    case 'manual':
      {
        handleUnblock(step, finish);
        break;
      }
    case 'prompt':
      {
        handlePrompt(step, finish);
      }
  }
}

function logOutput(name, output) {
  var ok = arguments.length <= 2 || arguments[2] === undefined ? true : arguments[2];

  output.split('\n').forEach(function (str) {
    console.log(('[' + new Date() + ' ' + name + ']')[ok ? 'green' : 'red'], str);
  });
}

function runCommand(i, step, done) {
  var proc = _child_process2.default.exec(step.command, {
    env: (0, _merge2.default)(process.env, file.env || {})
  });

  var spinner = new _ioSpin2.default('Box1', step.name);

  spinner.start();

  proc.stdout.on('data', function (data) {
    spinner.stop();
    logOutput(step.name, data.toString().replace(/\n$/, ''), true);
    spinner.start();
  });

  proc.stderr.on('data', function (data) {
    spinner.stop();
    logOutput(step.name, data.toString().replace(/\n$/, ''), false);
    spinner.start();
  });

  proc.on('exit', function (code) {
    spinner.stop();
    if (code) {
      logOutput(step.name, 'Exited with code ' + code, false);
      process.exit();
    }
    checkForNewPipeline(i);
    done();
  });
}

function handleWait(step, done) {
  done();
}

function handleUnblock(step, done) {
  var message = 'Would you like to unblock the next step?';

  if (step.label) {
    message = step.label;
  }

  (0, _promptForConfirm2.default)(message, function (ok) {
    if (!ok) {
      process.exit();
    }

    return done();
  });
}

function handlePrompt(step, done) {
  var message = 'Would you like to unblock the next step?';

  if (step.block) {
    message = step.block;
  }

  (0, _promptForConfirm2.default)(message, function (ok) {
    if (!ok) {
      process.exit();
    }

    var questions = step.fields.map(function (field) {
      if (field.options) {
        return {
          type: 'list',
          name: field.key,
          message: field.select,
          choices: field.options
        };
      }
      if (field.text) {
        return {
          type: 'input',
          name: field.key,
          message: field.text,
          default: field.default
        };
      }
    });

    _inquirer2.default.prompt(questions).then(function (answers) {
      for (var k in answers) {
        (0, _metadata.setMetadata)(buildId, k, answers[k]);
      }
      done();
    });
  });
}

function rewriteStep(step) {
  if (!step.command) {
    return step;
  }
  step.command = step.command.replace(/buildkite-agent/g, 'bkrun-agent ' + buildId);
  return step;
}

function checkForNewPipeline(stepIndex) {
  var response = (0, _pipeline.getPipeline)(buildId);
  if (!response) {
    return;
  }

  var replace = response.replace;
  var pipeline = response.pipeline;

  // Merge new environment vars

  file.env = (0, _merge2.default)(file.env, pipeline.env || {});

  for (var i = 0; i < pipeline.steps.length; i++) {
    var step = pipeline.steps[i];
    file.steps.splice(stepIndex + 1 + i, 0, step);
  }

  if (replace) {
    var count = stepIndex + pipeline.steps.length + 1;
    file.steps.splice(count, file.steps.length - count);
  }

  (0, _pipeline.deletePipeline)(buildId);
}

processStep(startIndex);